#include <MIDI.h>;
#include "Controller.h"
#include <FastLED.h>

/*************************************************************
  MIDI CONTROLLER

  by Notes and Volts
  www.notesandvolts.com

  Version 1.2 **Arduino UNO ONLY!**
  Without Multiplexer it will work on Mega
 *************************************************************/

MIDI_CREATE_DEFAULT_INSTANCE();
#define My_Channel 10
#define NUM_STRIP 1
#define NUM_LEDS_PER_STRIP 12
#define My_BRIGHTNESS  15
CRGB leds[NUM_STRIP * NUM_LEDS_PER_STRIP];

//************************************************************
//***SET THE NUMBER OF CONTROLS USED**************************
//************************************************************
//---How many buttons are connected directly to pins?---------
byte NUMBER_BUTTONS = 19;
//---How many potentiometers are connected directly to pins?--
byte NUMBER_POTS = 12;
//---How many buttons are connected to a multiplexer?---------
byte NUMBER_MUX_BUTTONS = 0;
//---How many potentiometers are connected to a multiplexer?--
byte NUMBER_MUX_POTS = 0;
//************************************************************

//***ANY MULTIPLEXERS? (74HC4067)************************************
//MUX address pins must be connected to Arduino UNO pins 2,3,4,5
//A0 = PIN2, A1 = PIN3, A2 = PIN4, A3 = PIN5
//*******************************************************************
//Mux NAME (OUTPUT PIN, , How Many Mux Pins?(8 or 16) , Is It Analog?);


//Mux M1(10, 16, false); //Digital multiplexer on Arduino pin 10
//Mux M2(A5, 8, true); //Analog multiplexer on Arduino analog pin A0
//*******************************************************************


//***DEFINE DIRECTLY CONNECTED POTENTIOMETERS************************
//Pot (Pin Number, Command, CC Control, Channel Number)
//**Command parameter is for future use**

Pot PO1(A1, 0, 1, 2);
Pot PO2(A2, 0, 2, 2);
Pot PO3(A3, 0, 3, 2);
Pot PO4(A4, 0, 4, 2);
Pot PO5(A5, 0, 5, 2);
Pot PO6(A6, 0, 6, 2);
Pot PO7(A7, 0, 7, 2);
Pot PO8(A8, 0, 8, 2);
Pot PO9(A9, 0, 9, 2);
Pot P10(A10, 0, 10, 2);
Pot P11(A11, 0, 11, 2);
Pot P12(A12, 0, 12, 2);
//*******************************************************************
//Add pots used to array below like this->  Pot *POTS[] {&PO1, &PO2, &PO3, &PO4, &PO5, &PO6};
Pot *POTS[] {&PO1, &PO2, &PO3, &PO4, &PO5, &PO6, &PO7, &PO8, &PO9, &P10, &P11, &P12};
//*******************************************************************
// &PO1, &PO2, &PO3, &PO4, &PO5, &PO6, &PO7, &PO8, &PO9, &P10, &P11, &P12

//***DEFINE DIRECTLY CONNECTED BUTTONS*******************************
//Button (Pin Number, Command, Note Number, Channel, Debounce Time)
//** Command parameter 0=NOTE  1=CC  2=Toggle CC **

Button BU15(15, 0, 15, 2, 10 );
Button BU16(16, 0, 16, 2, 10 );
Button BU17(17, 0, 17, 2, 10 );
Button BU18(18, 0, 18, 2, 10 );
Button BU19(19, 0, 19, 2, 10 );
Button BU20(20, 0, 20, 2, 10 );
Button BU21(21, 0, 21, 2, 10 );
Button BU22(22, 0, 22, 2, 10 );
Button BU23(23, 0, 23, 2, 10 );
Button BU24(24, 0, 24, 2, 10 );
Button BU26(26, 0, 26, 2, 10 );
Button BU27(27, 0, 27, 2, 10 );
Button BU28(28, 0, 28, 2, 10 );
Button BU29(29, 0, 29, 2, 10 );
Button BU33(33, 0, 33, 2, 10 );
Button BU34(34, 0, 34, 2, 10 );
Button BU35(35, 0, 35, 2, 10 );
Button BU36(36, 0, 36, 2, 10 );
Button BU37(37, 0, 37, 2, 10 );

//*******************************************************************
//Add buttons used to array below like this->  Button *BUTTONS[] {&BU1, &BU2, &BU3, &BU4, &BU5, &BU6, &BU7, &BU8};
Button *BUTTONS[] {&BU15, &BU16, &BU17, &BU18, &BU19, &BU20, &BU21, &BU22, &BU23, &BU24, &BU26, &BU27, &BU28, &BU29, &BU33, &BU34, &BU35, &BU36, &BU37};
//*******************************************************************


//***DEFINE BUTTONS CONNECTED TO MULTIPLEXER*************************
//Button::Button(Mux mux, byte muxpin, byte command, byte value, byte channel, byte debounce)
//** Command parameter 0=NOTE  1=CC  2=Toggle CC **

//Button MBU1(M1, 0, 0, 70, 1, 5);
//Button MBU2(M1, 1, 1, 71, 1, 5);
//Button MBU3(M1, 2, 2, 72, 1, 5);
//Button MBU4(M1, 3, 0, 73, 1, 5);
//Button MBU5(M1, 4, 0, 74, 1, 5);
//Button MBU6(M1, 5, 0, 75, 1, 5);
//Button MBU7(M1, 6, 0, 76, 1, 5);
//Button MBU8(M1, 7, 0, 77, 1, 5);
//Button MBU9(M1, 8, 0, 78, 1, 5);
//Button MBU10(M1, 9, 0, 79, 1, 5);
//Button MBU11(M1, 10, 0, 80, 1, 5);
//Button MBU12(M1, 11, 0, 81, 1, 5);
//Button MBU13(M1, 12, 0, 82, 1, 5);
//Button MBU14(M1, 13, 0, 83, 1, 5);
//Button MBU15(M1, 14, 0, 84, 1, 5);
//Button MBU16(M1, 15, 0, 85, 1, 5);
//*******************************************************************
////Add multiplexed buttons used to array below like this->  Button *MUXBUTTONS[] {&MBU1, &MBU2, &MBU3, &MBU4, &MBU5, &MBU6.....};
Button *MUXBUTTONS[] {};

//*******************************************************************


//***DEFINE POTENTIOMETERS CONNECTED TO MULTIPLEXER*******************
//Pot::Pot(Mux mux, byte muxpin, byte command, byte control, byte channel)
//**Command parameter is for future use**

//Pot MPO1(M2, 0, 0, 1, 1);
//Pot MPO2(M2, 1, 0, 7, 1);
//Pot MPO3(M2, 2, 0, 50, 1);
//Pot MPO4(M2, 3, 0, 55, 2);
//Pot MPO5(M2, 4, 0, 50, 1);
//Pot MPO6(M2, 5, 0, 55, 2);
//Pot MPO7(M2, 6, 0, 50, 1);
//Pot MPO8(M2, 7, 0, 55, 2);
//Pot MPO9(M2, 8, 0, 50, 1);
//Pot MPO10(M2, 9, 0, 55, 2);
//Pot MPO11(M2, 10, 0, 50, 1);
//Pot MPO12(M2, 11, 0, 55, 2);
//Pot MPO13(M2, 12, 0, 50, 1);
//Pot MPO14(M2, 13, 0, 55, 2);
//Pot MPO15(M2, 14, 0, 50, 1);
//Pot MPO16(M2, 15, 0, 55, 2);
//*******************************************************************
//Add multiplexed pots used to array below like this->  Pot *MUXPOTS[] {&MPO1, &MPO2, &MPO3, &MPO4, &MPO5, &MPO6.....};
Pot *MUXPOTS[] {};
//*******************************************************************


void setup() {
  MIDI.begin(10); // On Channel 10
  //Serial.begin(38400);
  Serial.begin(115200);
  pinMode(LED_BUILTIN, OUTPUT);
  for (int a = 3; a <= 10; a++) {
    pinMode(a, OUTPUT);
    digitalWrite(a, LOW);
  }

  FastLED.addLeds<NEOPIXEL, 2>(leds, 0, NUM_LEDS_PER_STRIP); // RGB strip on D2
  FastLED.setBrightness(My_BRIGHTNESS );
}

void loop() {
  if (MIDI.read())                    // If we have received a message
    {
      if (MIDI.getChannel() == My_Channel) {
      switch (MIDI.getData1()) {
        //******** Track 1 ********//
        case 11:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[0] = CRGB::Red;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[0] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 12:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[1] = CRGB::White;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[1] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 13:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[2] = CRGB::Green;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[2] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 14:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[2] = CRGB::Blue;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[2] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 15:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[1] = CRGB::Orange;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[1] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 16:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            digitalWrite(7, HIGH);
          }
          else if (MIDI.getData2() == 0) {
            digitalWrite(7, LOW);
          }
          break;
        //******** Track 2 ********//
        case 21:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[3] = CRGB::Red;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[3] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 22:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[4] = CRGB::White;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[4] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 23:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[5] = CRGB::Green;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[5] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 24:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[5] = CRGB::Blue;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[5] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 25:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[4] = CRGB::Orange;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[4] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 26:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            digitalWrite(8, HIGH);
          }
          else if (MIDI.getData2() == 0) {
            digitalWrite(8, LOW);
          }
          break;
              
        //******** Track 3 ********//
        case 31:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[6] = CRGB::Red;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[6] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 32:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[7] = CRGB::White;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[7] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 33:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[8] = CRGB::Green;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[8] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 34:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[8] = CRGB::Blue;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[8] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 35:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[7] = CRGB::Orange;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[7] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 36:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            digitalWrite(9, HIGH);
          }
          else if (MIDI.getData2() == 0) {
            digitalWrite(9, LOW);
          }
          break;
              
        //******** Track 4 ********//
        case 41:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[9] = CRGB::Red;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[9] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 42:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[10] = CRGB::White;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[10] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 43:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[11] = CRGB::Green;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[11] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 44:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[11] = CRGB::Blue;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[11] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 45:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[10] = CRGB::Orange;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[10] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 46:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            digitalWrite(10, HIGH);
          }
          else if (MIDI.getData2() == 0) {
            digitalWrite(10, LOW);
          }
          break;
        //******** Global ********//
        case 50:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            digitalWrite(4, HIGH);
          }
          else if (MIDI.getData2() == 0) {
            digitalWrite(4, LOW);
          }
          break;
        case 55:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            digitalWrite(5, HIGH);
          }
          else if (MIDI.getData2() == 0) {
            digitalWrite(5, LOW);
          }
          break;
        case 56:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            digitalWrite(6, HIGH);
          }
          else if (MIDI.getData2() == 0) {
            digitalWrite(6, LOW);
          }
          break;
        case 70:
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            digitalWrite(3, HIGH);
          }
          else if (MIDI.getData2() == 0) {
            digitalWrite(3, LOW);
          }
          break;
        
        //******** Song LED ********//
          //case 100:
          //for (int b = 0; b <= 11; b++) {
          //  leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          //if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
          //  leds[0] = CRGB::Black;
          //  FastLED.show();
          //}
          //else if (MIDI.getData2() == 0) {
          //  leds[0] = CRGB::Black;
          //  FastLED.show();
          //}
          //break;
        case 101:
          //for (int b = 0; b <= 11; b++) {
          //  leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[0] = CRGB::Cyan;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[0] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 102:
          //for (int b = 0; b <= 11; b++) {
          // leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[1] = CRGB::Cyan;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[1] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 103:
          //for (int b = 0; b <= 11; b++) {
          //  leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[2] = CRGB::Cyan;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[2] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 104:
          //for (int b = 0; b <= 11; b++) {
          //  leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[3] = CRGB::Cyan;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[3] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 105:
          //for (int b = 0; b <= 11; b++) {
          //  leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[4] = CRGB::Cyan;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[4] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 106:
          //for (int b = 0; b <= 11; b++) {
          //  leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[5] = CRGB::Cyan;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[5] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 107:
          //for (int b = 0; b <= 11; b++) {
          //  leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[6] = CRGB::Cyan;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[6] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 108:
          //for (int b = 0; b <= 11; b++) {
          //  leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[7] = CRGB::Cyan;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[7] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 109:
          //for (int b = 0; b <= 11; b++) {
          //  leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[8] = CRGB::Cyan;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[8] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 110:
          //for (int b = 0; b <= 11; b++) {
          //  leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[9] = CRGB::Cyan;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[9] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 111:
          //for (int b = 0; b <= 11; b++) {
           // leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[10] = CRGB::Cyan;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[10] = CRGB::Black;
            FastLED.show();
          }
          break;
        case 112:
          //for (int b = 0; b <= 11; b++) {
          //  leds[b] = CRGB::Black;
          //  FastLED.show();
          //}
          if (MIDI.getType() == midi::NoteOn && MIDI.getData2() == 127) {
            leds[11] = CRGB::White;
            FastLED.show();
          }
          else if (MIDI.getData2() == 0) {
            leds[11] = CRGB::Black;
            FastLED.show();
          }
          break;
      }
    }
  };
  if (NUMBER_BUTTONS != 0) updateButtons();
  if (NUMBER_POTS != 0) updatePots();
  if (NUMBER_MUX_BUTTONS != 0) updateMuxButtons();
  if (NUMBER_MUX_POTS != 0) updateMuxPots();
}


//*****************************************************************
void updateButtons() {

  // Cycle through Button array
  for (int i = 0; i < NUMBER_BUTTONS; i = i + 1) {
    byte message = BUTTONS[i]->getValue();

    //  Button is pressed
    if (message == 0) {
      switch (BUTTONS[i]->Bcommand) {
        case 0: //Note
          MIDI.sendNoteOn(BUTTONS[i]->Bvalue, 127, BUTTONS[i]->Bchannel);
          break;
        case 1: //CC
          MIDI.sendControlChange(BUTTONS[i]->Bvalue, 127, BUTTONS[i]->Bchannel);
          break;
        case 2: //Toggle
          if (BUTTONS[i]->Btoggle == 0) {
            MIDI.sendControlChange(BUTTONS[i]->Bvalue, 127, BUTTONS[i]->Bchannel);
            BUTTONS[i]->Btoggle = 1;
          }
          else if (BUTTONS[i]->Btoggle == 1) {
            MIDI.sendControlChange(BUTTONS[i]->Bvalue, 0, BUTTONS[i]->Bchannel);
            BUTTONS[i]->Btoggle = 0;
          }
          break;
      }
    }

    //  Button is not pressed
    if (message == 1) {
      switch (BUTTONS[i]->Bcommand) {
        case 0:
          MIDI.sendNoteOff(BUTTONS[i]->Bvalue, 0, BUTTONS[i]->Bchannel);
          break;
        case 1:
          MIDI.sendControlChange(BUTTONS[i]->Bvalue, 0, BUTTONS[i]->Bchannel);
          break;
      }
    }
  }
}
//*******************************************************************
void updateMuxButtons() {

  // Cycle through Mux Button array
  for (int i = 0; i < NUMBER_MUX_BUTTONS; i = i + 1) {

    MUXBUTTONS[i]->muxUpdate();
    byte message = MUXBUTTONS[i]->getValue();

    //  Button is pressed
    if (message == 0) {
      switch (MUXBUTTONS[i]->Bcommand) {
        case 0: //Note
          MIDI.sendNoteOn(MUXBUTTONS[i]->Bvalue, 127, MUXBUTTONS[i]->Bchannel);
          break;
        case 1: //CC
          MIDI.sendControlChange(MUXBUTTONS[i]->Bvalue, 127, MUXBUTTONS[i]->Bchannel);
          break;
        case 2: //Toggle
          if (MUXBUTTONS[i]->Btoggle == 0) {
            MIDI.sendControlChange(MUXBUTTONS[i]->Bvalue, 127, MUXBUTTONS[i]->Bchannel);
            MUXBUTTONS[i]->Btoggle = 1;
          }
          else if (MUXBUTTONS[i]->Btoggle == 1) {
            MIDI.sendControlChange(MUXBUTTONS[i]->Bvalue, 0, MUXBUTTONS[i]->Bchannel);
            MUXBUTTONS[i]->Btoggle = 0;
          }
          break;
      }
    }
    //  Button is not pressed
    if (message == 1) {
      switch (MUXBUTTONS[i]->Bcommand) {
        case 0:
          MIDI.sendNoteOff(MUXBUTTONS[i]->Bvalue, 0, MUXBUTTONS[i]->Bchannel);
          break;
        case 1:
          MIDI.sendControlChange(MUXBUTTONS[i]->Bvalue, 0, MUXBUTTONS[i]->Bchannel);
          break;
      }
    }
  }
}
//***********************************************************************
void updatePots() {
  for (int i = 0; i < NUMBER_POTS; i = i + 1) {
    byte potmessage = POTS[i]->getValue();
    if (potmessage != 255) MIDI.sendControlChange(POTS[i]->Pcontrol, potmessage, POTS[i]->Pchannel);
  }
}
//***********************************************************************
void updateMuxPots() {
  for (int i = 0; i < NUMBER_MUX_POTS; i = i + 1) {
    MUXPOTS[i]->muxUpdate();
    byte potmessage = MUXPOTS[i]->getValue();
    if (potmessage != 255) MIDI.sendControlChange(MUXPOTS[i]->Pcontrol, potmessage, MUXPOTS[i]->Pchannel);
  }
}
